;; RedMonk/theme.jl
;; Copyright (C) 2000 Unai Uribarri <unaiur@telecable.es>

;; This file is part of sawmill.

;; sawmill is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; sawmill is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with sawmill; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(require 'gradient)

(let*
    ( 
     ;; Icon masks (9x9)
     (menu-mask  [ 
		 [ () () () () () () () () () ]
		 [ t  t  t  t  t  t  t  t  t  ]
		 [ t  t  t  t  t  t  t  t  t  ]
		 [ () () () () () () () () () ]
		 [ () t  t  t  t  t  t  t  () ]
		 [ () () t  t  t  t  t  () () ]
		 [ () () () t  t  t  () () () ]
		 [ () () () () t  () () () () ]
		 [ () () () () () () () () () ]])
     
     (close-mask [ 
		 [ () () () () () () () () () ]
		 [ () t  t  () () () () t  t  ]
		 [ () () t  t  () () t  t  () ]
		 [ () () () t  t  t  t  () () ]
		 [ () () () () t  t  () () () ]
		 [ () () () t  t  t  t  () () ]
		 [ () () t  t  () () t  t  () ]
		 [ () t  t  () () () () t  t  ]
		 [ () () () () () () () () () ]])
     
     (max-mask   [
		 [ t  t  t  t  t  t  t  t  t  ]
		 [ t  t  t  t  t  t  t  t  t  ]
		 [ t  () () () () () () () t  ]
		 [ t  () () () () () () () t  ]
		 [ t  () () () () () () () t  ]
		 [ t  () () () () () () () t  ]
		 [ t  () () () () () () () t  ]
		 [ t  () () () () () () () t  ]
		 [ t  t  t  t  t  t  t  t  t  ]])
     
     (rest-mask  [
		 [ () () t  t  t  t  t  t  () ]
		 [ () () t  t  t  t  t  t  () ]
		 [ () () t  () () () () t  () ]
		 [ t  t  t  t  t  t  () t  () ]
		 [ t  t  t  t  t  t  () t  () ]
		 [ t  () () () () t  t  t  () ]
		 [ t  () () () () t  () () () ]
		 [ t  () () () () t  () () () ]
		 [ t  t  t  t  t  t  () () () ]])
     
     (min-mask   [
		 [ () () () () () () () () () ]
		 [ () () () () () () () () () ]
		 [ () () () () () () () () () ]
		 [ () () () () () () () () () ]
		 [ () () () () () () () () () ]
		 [ () () () () () () () () () ]
		 [ () t  t  t  t  t  t  () () ]
		 [ () t  t  t  t  t  t  () () ]
		 [ () () () () () () () () () ]])
     
     ;; Color definitions ( black white darkgray lightgray title )
     (Windows-95 
      (let ((rgb get-color-rgb))
	(vector (rgb #x0000 #x0000 #x0000) (rgb #xffff #xffff #xffff) 
		(rgb #x8000 #x8000 #x8000) (rgb #xC000 #xC000 #xC000) 
		(cons (rgb #x8000 #x8000 #x8000) (rgb #x8000 #x8000 #x8000))
		(cons (rgb #x0000 #x0000 #x8000) (rgb #x0000 #x0000 #x8000)))))
     (Windows-2000 
      (let ((rgb get-color-rgb))
	(vector (rgb #x4000 #x4000 #x4000) (rgb #xffff #xffff #xffff) 
		(rgb #x8000 #x8000 #x8000) (rgb #xD400 #xD000 #xC800) 
		(cons (rgb #x8000 #x8000 #x8000) (rgb #xC000 #xC000 #xC000))
		(cons (rgb #x0A00 #x2400 #x6A00) (rgb #xA600 #xCA00 #xF000)))))

     (Bloodlust 
      (let ((rgb get-color-rgb))
	(vector (rgb #x4000 #x0000 #x0000) (rgb #xFF00 #xC800 #xC800) 
		(rgb #x8000 #x0000 #x0000) (rgb #xD800 #x7800 #x7000) 
		(cons (rgb #x6000 #x4C00 #x6000) (rgb #x0000 #x4C00 #x0000))
		(cons (rgb #xA000 #x3000 #x0A00) (rgb #x4000 #x3000 #x0000)))))

     (GreenLand 
      (let ((rgb get-color-rgb))
	(vector (rgb #x4000 #x4000 #x4000) (rgb #xffff #xffff #xffff) 
		(rgb #x8000 #x8000 #x8000) (rgb #xD400 #xD000 #xC800) 
		(cons (rgb #x8000 #x8000 #x8000) (rgb #xC000 #xC000 #xC000))
		(cons (rgb #x0A00 #x6800 #x0A00) (rgb #xA600 #xF000 #xA600)))))


     (colors GreenLand)

     ;; Color accesors
     (black  (lambda () (aref colors 0)))
     (white  (lambda () (aref colors 1)))
     (dgray  (lambda () (aref colors 2)))
     (lgray  (lambda () (aref colors 3)))
     (dtitle (lambda () (aref colors 4)))
     (ltitle (lambda () (aref colors 5)))

     ;; Convert a color object to a ( R G B )/256 list
     (rgb (lambda (color)
	    (mapcar (lambda (c) (quotient c 256)) (color-rgb color))))

     ;; Button rendering
     (make-button (lambda (im mask up)
		    (let ((icon-up-gen 
			   (lambda (x y) 
			     (rgb
			      (cond
			       ((or (= x 15) (= y 13)) (black))
			       ((or (= x 0)  (= y 0))  (white))
			       ((or (= x 14) (= y 12)) (dgray))
			       ((and (>= x 3) (>= y 2) (< x 12) (< y 11))
				(if (aref (aref mask (- y 2)) (- x 3))
				    (black) (lgray)))
			       (t (lgray))))))
			  (icon-dn-gen 
			   (lambda (x y)
			     (rgb
			      (cond
			       ((or (= x 15) (= y 13)) (white))
			       ((or (= x 0)  (= y 0))  (black))
			       ((or (= x 14) (= y 12)) (lgray))
			       ((or (= x 1)  (= y 1))  (dgray))
			       ((and (>= x 4) (>= y 3) (< x 13) (< y 12))
				(if (aref (aref mask (- y 3)) (- x 4))
				    (black) (lgray)))
			       (t (lgray)))))))
		      (image-fill (if up icon-up-gen icon-dn-gen) im)
		      im)))

					   
     
     ;; images
     ;; gradients
     (dtitle-grad    (make-sized-image (screen-width) 1))
     (ltitle-grad    (make-sized-image (screen-width) 1))
     
     ;; buttons
     (close-btn-up   (make-sized-image 16 14))
     (close-btn-down (make-sized-image 16 14))
     (min-btn-up     (make-sized-image 16 14))
     (min-btn-down   (make-sized-image 16 14))
     (max-btn-up     (make-sized-image 16 14))
     (max-btn-down   (make-sized-image 16 14))
     (rest-btn-up    (make-sized-image 16 14))
     (rest-btn-down  (make-sized-image 16 14))
     (menu-btn-up    (make-sized-image 16 14))
     (menu-btn-down  (make-sized-image 16 14))
     
     ;; borders
     (tborder      (make-sized-image 1 4)) ; top border
     (bborder      (make-sized-image 1 4)) ; bottom border
     (lborder      (make-sized-image 4 1)) ; left border
     (rborder      (make-sized-image 4 1)) ; right border
     
     ;; corners
     (tlcorner     (make-sized-image 4 4)) 
     (trcorner     (make-sized-image 4 4)) 
     (blcorner     (make-sized-image 4 4)) 
     (brcorner     (make-sized-image 4 4)) 
     
     ;; Image rendering
     (rebuild-images 
      (lambda ()
	(let ((image-setter 
	       (lambda (img color)
		 (lambda (point) 
		   (image-set img (car point) (cdr point) (rgb color)))))
	      (grad-setter
	       (lambda (img color)
		 (draw-horizontal-gradient img (car color) (cdr color)))))
		     
	  
	  ;; gradients
	  (grad-setter dtitle-grad (dtitle))
	  (grad-setter ltitle-grad (ltitle))
	  
	  ;; buttons
	  (make-button close-btn-up   close-mask t)
	  (make-button close-btn-down close-mask nil)
	  (make-button min-btn-up     min-mask t)
	  (make-button min-btn-down   min-mask nil)
	  (make-button max-btn-up     max-mask t)
	  (make-button max-btn-down   max-mask nil)
	  (make-button rest-btn-up    rest-mask t)
	  (make-button rest-btn-down  rest-mask nil)
	  (make-button menu-btn-up    menu-mask t)
	  (make-button menu-btn-down  menu-mask nil)

	  ;; borders
	  (clear-image tborder (lgray))
	  (image-set tborder 0 1 (rgb (white)))
	  
	  (clear-image bborder (lgray))
	  (image-set bborder 0 2 (rgb (dgray)))
	  (image-set bborder 0 3 (rgb (black)))
	  
	  (clear-image lborder (lgray))
	  (image-set lborder 1 0 (rgb (white)))

	  (clear-image rborder (lgray))
	  (image-set rborder 2 0 (rgb (dgray)))
	  (image-set rborder 3 0 (rgb (black)))
		     
	  ;; corners
	  (clear-image tlcorner (lgray))
	  (mapc (image-setter tlcorner (white))
		'((1 . 1)(1 . 2)(1 . 3)(2 . 1)(3 . 1)))

	  (clear-image trcorner (lgray))
	  (mapc (image-setter trcorner (white))
		'((0 . 1)(1 . 1)))
	  (mapc (image-setter trcorner (black))
		'((3 . 0)(3 . 1)(3 . 2)(3 . 3)))
	  (mapc (image-setter trcorner (dgray))
		'((2 . 1)(2 . 2)(2 . 3)))

	  (clear-image blcorner (lgray))
	  (mapc (image-setter blcorner (white))
		'((1 . 0)(1 . 1)))
	  (mapc (image-setter blcorner (black))
		'((0 . 3)(1 . 3)(2 . 3)(3 . 3)))
	  (mapc (image-setter blcorner (dgray))
		'((1 . 2)(2 . 2)(3 . 2)))
	  
	  (clear-image brcorner (black))
	  (mapc (image-setter brcorner (dgray))
		'((0 . 2)(1 . 2)(2 . 2)(2 . 1)(2 . 0)))
	  (mapc (image-setter brcorner (lgray))
		'((0 . 0)(0 . 1)(1 . 0)(1 . 1))))))

     (make-icon (lambda (win)
		  (let ((icon (window-icon-image win)))
		    (if (not icon)
			(list menu-btn-up nil nil menu-btn-down)
		      (scale-image icon 18 18)))))

     (maximize-hook (lambda (win)
		      (if (eq 'RedMonk (window-get win 'current-frame-style))
			  (reframe-window win))))

     (box-frame `(;; top-border
		  ((background  . ,tborder)
		   (top-edge    . -23)
		   (left-edge   . 0)
		   (right-edge  . 0)
		   (height      . 4)
		   (class       . top-border))
		  ;; bottom-border
		  ((background  . ,bborder)
		   (bottom-edge . -4)
		   (left-edge   . 0)
		   (right-edge  . 0)
		   (height      . 4)
		   (class       . bottom-border))
		  ;; left-border
		  ((background  . ,lborder)
		   (top-edge    . -19)
		   (bottom-edge . 0)
		   (left-edge   . -4)
		   (width       . 4)
		   (class       . left-border))
		  ;; right-border
		  ((background  . ,rborder)
		   (top-edge    . -19)
		   (bottom-edge . 0)
		   (right-edge . -4)
		   (width      . 4)
		   (class      . right-border))
		  ;; top-left-corner
		  ((background . ,tlcorner)
		   (top-edge   . -23)
		   (left-edge  . -4)
		   (height     . 4)
		   (width      . 4)
		   (class      . top-left-corner))
		  ;; top-right-corner
		  ((background . ,trcorner)
		   (top-edge   . -23)
		   (right-edge . -4)
		   (height     . 4)
		   (width      . 4)
		   (class      . top-right-corner))
		  ;; buttom-left-corner
		  ((background  . ,blcorner)
		   (bottom-edge . -4)
		   (left-edge   . -4)
		   (height      . 4)
		   (width       . 4)
		   (class       . bottom-left-corner))
		  ;; buttom-left-corner
		  ((background  . ,brcorner)
		   (bottom-edge . -4)
		   (right-edge  . -4)
		   (height      . 4)
		   (width       . 4)
		   (class       . bottom-right-corner)))))
  
  (defgroup RedMonk "RedMonk Theme"
    :group appearance)
  
  (defcustom RedMonk:os nil
    "Color map"
    :type (set Windows-2000 Windows-95 Bloodlust GreenLand)
    :group (appearance RedMonk)
    :user-level novice
    :after-set (lambda ()
		 (setq colors (symbol-value RedMonk:os))
		 (rebuild-images)
		 (reframe-windows-with-style 'RedMonk)))
  
  (rebuild-images)
  
  (add-frame-style 
   'RedMonk 
   (lambda (win type)
     (let ((show-btns (or (eq type `default) (eq type `shaped))) frame)
       (if (or (eq type `default) (eq type `transient))
	   (setq frame box-frame))
       (if show-btns
	   (setq frame 
		 `(;;Buttons background
		   ((background . ,(list (cdr (dtitle)) (cdr (ltitle))))
		    (top-edge   . -19)
		    (right-edge . 0)
		    (width      . 52)
		    (height     . 18))
		   ;;Close
		   ((background . ,(list (cdr (dtitle)) (cdr (ltitle))))
		    (foreground . ,(list close-btn-up nil nil close-btn-down))
		    (x-justify  . center)
		    (y-justify  . center)
		    (top-edge   . -17)
		    (right-edge . 2)
		    (width      . 16)
		    (height     . 14)
		    (removable  . true)
		    (class      . close-button))
		   ;;Maximize-Restore
		   ((background . (,(cdr (dtitle)) ,(cdr (ltitle))))
		    (foreground . ,(if (window-maximized-p win)
				       (list rest-btn-up nil nil rest-btn-down)
				     (list max-btn-up nil nil max-btn-down)))
		    (y-justify  . center)
		    (top-edge   . -17)
		    (right-edge . 20)
		    (width      . 16)
		    (height     . 14)
		    (removable  . true)
		    (class      . maximize-button))
		   ;;Iconify
		   ((background . ,(list (cdr (dtitle)) (cdr (ltitle))))
		    (foreground . ,(list min-btn-up nil nil min-btn-down))
		    (y-justify  . center)
		    (top-edge   . -17)
		    (right-edge . 36)
		    (width      . 16)
		    (height     . 14)
		    (removable  . true)
		    (class      . iconify-button))
		   . ,frame)))
       
       ;; title-part
       `(((background . (,dtitle-grad ,ltitle-grad))
	  (foreground . (,(lgray) ,(white)))
	  (text       . ,window-name)
	  (x-justify  . 2)
	  (y-justify  . center)
	  (left-edge  . 20)
	  (right-edge . ,(if show-btns 52 0))
	  (top-edge   . -19)
	  (height     . 18)
	  (class      . title))
	 ;; subtitle-part
	 ((background . ,(lgray))
	  (left-edge  . 0)
	  (right-edge . 0)
	  (top-edge   . -1)
	  (height     . 1))
	 ;; icon part
	 ((background . (,(car (dtitle)) ,(car (ltitle))))
	  (foreground . ,(make-icon win))
	  (x-justify  . center)
	  (y-justify  . center)
	  (top-edge   . -19)
	  (left-edge  . 0)
	  (width      . 20)
	  (height     . 18)
	  (class      . menu-button))
	 . ,frame))))


  ;; Hooks
  (call-after-property-changed 
   'WM_NAME (lambda (win) (rebuild-frame win)))
  (call-after-property-changed
   'WM_HINTS (lambda (win) (reframe-window win)))
  (add-hook 'window-maximized-hook maximize-hook)
  (add-hook 'window-unmaximized-hook maximize-hook))
